#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-08-01
#
# monitor Check Point anti malware
#
# 2017-06-17: added last update check
# 2017-01-03: fixed inventory function for bad ino data
# 2017-01-03: fixed lastupdate check
# 2018-03-08: fixed lastuodate check (again :-(), changed snmp scan function
# 2018-03-15: code cleanup
# 2020-06-07: code cleanup
# 2020-06-08: changed snmp-scan function
# 2021-08-14: rewritten for CMK 2.0
#
# sample info
# [
#  [[u'valid', u'Mon Jul  1 01:00:00 2019', u'Contract is up to date.', u'up-to-date',
#    u'Gateway is up to date. Database version: 1607290059. Package date: Thu Jul 28 01:00:00 2016.',
#    u'The next update will be run as scheduled.', u'1607290059']],
#  [[u'valid', u'Mon Jul  1 01:00:00 2019', u'Contract is up to date.', u'up-to-date',
#    u'Gateway is up to date. Database version: 1607290831. Package date: Thu Jul 28 01:00:00 2016.',
#    u'The next update will be run as scheduled.', u'1607290831']],
#  [[u'valid', u'Mon Jul  1 01:00:00 2019', u'Contract is up to date.']],
#  [[u'0', u'', u'', u'2229', u'4486', u'10000', u'0', u'7', u'24', u'113125']]]
#

import time
from dataclasses import dataclass
from typing import List, Dict, Any

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    check_levels,
    State,
    SNMPTree,
    all_of,
    startswith,
    any_of,
    equals,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)


@dataclass
class CheckpointAntibot:
    status: str
    expdate: str
    desc: str
    updatestatus: str
    updatedesc: str
    nextupdate: str
    version: str


@dataclass
class CheckpointAntivirus:
    status: str
    expdate: str
    desc: str
    updatestatus: str
    updatedesc: str
    nextupdate: str
    version: str


@dataclass
class CheckpointAntispam:
    status: str
    expdate: str
    desc: str


@dataclass
class CheckpointAntimalware:
    amwstauscode: int
    amwshortdesc: str
    amwlongdesc: str
    amw201: str
    amw202: str
    amw203: str
    amw211: str
    amw212: str
    amw213: str
    amw214: str


def _lastupdate_check(updateversion, timeformat):
    try:
        #  time.time() : time in seconds since the epoch (1970)
        #  time.strptime(updateversion[:6], '%y%m%d') : convert sting in time tuple
        #  time.mktime(time tuple) : convert time tuple in seconds since the epoch
        #  seconds/86400 : convert in days
        #  int(days) : cut flotingpoint
        lastupdate = int((time.time() - time.mktime(time.strptime(updateversion[:6], timeformat))) / 86400)
    except ValueError:
        return 'unknown'

    return lastupdate


def _parse_checkpoint_amw_anti_bot(antibot, amw: Dict):
    try:
        status, expdate, desc, updatestatus, updatedesc, nextupdate, version = antibot[0]
    except (ValueError, IndexError):
        return

    if len(version) != 0:
        amw['anti bot'] = CheckpointAntibot(
            status=status,
            expdate=expdate,
            desc=desc,
            updatestatus=updatestatus,
            updatedesc=updatedesc,
            nextupdate=nextupdate,
            version=version,
        )


def _parse_checkpoint_amw_anti_virus(antivirus, amw: Dict):
    try:
        status, expdate, desc, updatestatus, updatedesc, nextupdate, version = antivirus[0]
    except (ValueError, IndexError):
        return

    if status != '':
        amw['anti virus'] = CheckpointAntibot(
            status=status,
            expdate=expdate,
            desc=desc,
            updatestatus=updatestatus,
            updatedesc=updatedesc,
            nextupdate=nextupdate,
            version=version,
        )


def _parse_checkpoint_amw_anti_spam(antispam, amw: Dict):
    try:
        status, expdate, desc = antispam[0]
    except (ValueError, IndexError):
        return

    if status != '':
        amw['anti spam'] = CheckpointAntispam(
            status=status,
            expdate=expdate,
            desc=desc,
        )


def _parse_checkpoint_amw_anti_malware(overall, amw: Dict):
    try:
        amwstauscode, amwshortdesc, amwlongdesc, amw201, amw202, amw203, amw211, amw212, amw213, amw214 = overall[0]
    except (IndexError, ValueError):
        return

    amw['over all'] = CheckpointAntimalware(
        amwstauscode=int(amwstauscode),
        amwshortdesc=amwshortdesc,
        amwlongdesc=amwlongdesc,
        amw201=amw201,
        amw202=amw202,
        amw203=amw203,
        amw211=amw211,
        amw212=amw212,
        amw213=amw213,
        amw214=amw214,
    )


def parse_checkpoint_amw(string_table: List[StringTable]) -> Dict[str, Any]:
    amw = {}
    antibot, antivirus, antispam, overall = string_table

    _parse_checkpoint_amw_anti_bot(antibot, amw)
    _parse_checkpoint_amw_anti_virus(antivirus, amw)
    _parse_checkpoint_amw_anti_spam(antispam, amw)
    _parse_checkpoint_amw_anti_malware(overall, amw)

    return amw


def discovery_checkpoint_amw(section: Dict[str, Any]) -> DiscoveryResult:
    for key in section.keys():
        yield Service(item=key)


def check_checkpoint_amw(item, params, section: Dict[str, Any]) -> CheckResult:
    if item == 'anti bot':
        anti_bot: CheckpointAntibot = section[item]

        lastupdate = _lastupdate_check(anti_bot.version, params['timeformat'])

        if len(anti_bot.desc) >= 2:
            yield Result(state=State.OK, summary=anti_bot.desc)
        if len(anti_bot.expdate) >= 2:
            yield Result(state=State.OK, summary=f'Valid until: {anti_bot.expdate}')

        yield Result(state=State.OK, summary=f'Version: {anti_bot.version}')

        if anti_bot.status != 'valid':
            yield Result(state=State.WARN, summary=f'Status: {anti_bot.status}')

        if (anti_bot.updatestatus != 'up-to-date') and (anti_bot.updatestatus != 'new'):
            yield Result(state=State.WARN, summary=f'Update description: {anti_bot.updatedesc}')
        else:
            yield Result(state=State.OK, notice=f'Update description: {anti_bot.updatedesc}')

        if str(lastupdate).isdigit():
            yield from check_levels(
                value=lastupdate,
                label='Last update days ago',
                levels_upper=params['levels_no_update'],
                notice_only=True,
                render_func=lambda v: f'{v:.0f}',
            )
        else:
            yield Result(state=State.WARN, notice=f'Last update: {lastupdate}')

    if item == 'anti virus':
        anti_virus: CheckpointAntivirus = section[item]

        lastupdate = _lastupdate_check(anti_virus.version, params['timeformat'])

        if len(anti_virus.desc) >= 2:
            yield Result(state=State.OK, summary=anti_virus.desc)
        if len(anti_virus.expdate) >= 2:
            yield Result(state=State.OK, summary=f'Valid until: {anti_virus.expdate}')

        yield Result(state=State.OK, summary=f'Version: {anti_virus.version}')

        if anti_virus.status != 'valid':
            yield Result(state=State.WARN, summary=f'Status: {anti_virus.status}')

        if (anti_virus.updatestatus != 'up-to-date') and (anti_virus.updatestatus != 'new'):
            yield Result(state=State.WARN, notice=f'Update description: {anti_virus.updatedesc}')
        else:
            yield Result(state=State.OK, notice=f'Update description: {anti_virus.updatedesc}')

        if str(lastupdate).isdigit():
            yield from check_levels(
                value=lastupdate,
                label='Last update days ago',
                levels_upper=params['levels_no_update'],
                notice_only=True,
                render_func=lambda v: f'{v:.0f}',
            )
        else:
            yield Result(state=State.WARN, notice=f'Last update: {lastupdate}')

    if item == 'anti spam':
        anti_spam: CheckpointAntispam = section[item]

        if len(anti_spam.desc) >= 2:
            yield Result(state=State.OK, summary=anti_spam.desc)
        if len(anti_spam.expdate) >= 2:
            yield Result(state=State.OK, summary=f'Valid until: {anti_spam.expdate}')

        if anti_spam.status != 'valid':
            yield Result(state=State.WARN, summary=f'Status: {anti_spam.status}')

    if item == 'over all':
        anti_malware: CheckpointAntimalware = section[item]

        if len(anti_malware.amwlongdesc) != 0:
            yield Result(state=State.OK, notice=anti_malware.amwlongdesc)

        if anti_malware.amwstauscode != 0:
            yield Result(state=State.CRIT, summary=f'Status: {anti_malware.amwshortdesc}')
        else:
            yield Result(state=State.OK, summary='Status: running')

        yield Result(state=State.OK, notice=f'amw201: {anti_malware.amw201}')
        yield Result(state=State.OK, notice=f'amw202: {anti_malware.amw202}')
        yield Result(state=State.OK, notice=f'amw203: {anti_malware.amw203}')
        yield Result(state=State.OK, notice=f'amw211: {anti_malware.amw211}')
        yield Result(state=State.OK, notice=f'amw212: {anti_malware.amw212}')
        yield Result(state=State.OK, notice=f'amw213: {anti_malware.amw213}')
        yield Result(state=State.OK, notice=f'amw214: {anti_malware.amw214}')


register.snmp_section(
    name='checkpoint_amw',
    parse_function=parse_checkpoint_amw,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.46',  # CHECKPOINT-MIB::amw
            oids=[
                '2.1',  # antiBotSubscriptionStatus
                '2.2',  # antiBotSubscriptionExpDate
                '2.3',  # antiBotSubscriptionDesc
                '1.1',  # amwABUpdateStatus
                '1.2',  # amwABUpdateDesc
                '1.3',  # amwABNextUpdate
                '1.4',  # amwABVersion
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.46',  # CHECKPOINT-MIB::AntiVirus
            oids=[
                '3.1',  # antiVirusSubscriptionStatus
                '3.2',  # antiVirusSubscriptionExpDate
                '3.3',  # antiVirusSubscriptionDesc
                '5.1',  # amwAVUpdateStatus
                '5.2',  # amwAVUpdateDesc
                '5.3',  # amwAVNextUpdate
                '5.4',  # amwAVVersion
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.46.4',  # CHECKPOINT-MIB::AntiSpam
            oids=[
                '1',  # antiSpamSubscriptionStatus
                '2',  # antiSpamSubscriptionExpDate
                '3',  # antiSpamSubscriptionDesc
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.46',  # CHECKPOINT-MIB::OverallStaus
            oids=[
                '101',  # amwStatusCode
                '102',  # amwStatusShortDesc
                '103',  # amwStatusLongDesc
                '201',  # unknown: counter AB
                '202',  # unknown: counter AV
                '203',  # unknown: counter AS
                '211',  # unknown: counter
                '212',  # unknown: counter
                '213',  # unknown: counter
                '214',  # unknown: counter
            ]
        ),
    ],
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_amw',
    service_name='Anti malware: %s',
    discovery_function=discovery_checkpoint_amw,
    check_function=check_checkpoint_amw,
    check_ruleset_name='checkpoint_amw',
    check_default_parameters={
        'levels_no_update': (7, 14),
        'timeformat': '%y%m%d',
    }
)
